"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""


#!/usr/bin/python

"""
Autolabels names as (non-)ambiguous by looking for the phrase 'last_name (disambiguation)' on Wikipedia.
"""

import urllib2

URL = 'http://en.wikipedia.com/wiki/%s'

def get_name_label(name):
    try:
        the_url = URL % urllib2.quote(name.replace(" ", "_"))
        opener = urllib2.build_opener()
        opener.addheaders = [('User-agent', 'Mozilla/5.0')]
        data = opener.open(the_url)
        if type(data) != str:
            # Note that data.read() can only be done once.
            read_data = data.read()
            last_name = name.split()[-1]
            found = read_data.find("{} (disambiguation)".format(last_name)) != -1
            if found:
                found = read_data.find("{} (disambiguation)".format(name)) != -1
            else:
                found = read_data.find("disambiguation page".format(name)) != -1

            code = data.getcode()

            if code != 200:
                return 0
            else:
                return 1 if found else -1   # Ambiguous or non-ambiguous
        return 0
    except Exception as e:
        return 0

if __name__ == "__main__":

    import time
    import sys
    reload(sys)
    sys.setdefaultencoding("utf-8")
    import rfEntityId
    import extractNeighborhood

    if len(sys.argv) != 3:
        print "usage: {} adjlist_large.txt output_folder".format(sys.argv[0])
        sys.exit(1)

    adjlist_filename = sys.argv[1]
    output_folder = sys.argv[2]
    k=1

    adj_file = open(adjlist_filename, "r")
    for line in adj_file:
        time.sleep(1) # To keep off load from RF servers
        rf_id = line.split()[0]
        print "rf_id:", rf_id
        name = rfEntityId.getEntityName(rf_id, True) # Only look for persons
        if name == "":
            continue
        print "name:",name
        lbl = get_name_label(name)
        print "label",lbl
        if lbl in [-1, 1]:
            extractNeighborhood.extract_neighborhood(k, rf_id, "adjlist_large.txt", output_folder + "/{}.k{}.{}.txt".format(rf_id, k, lbl))

    adj_file.close()
