"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""

#!/usr/bin/python

import sys
import os
import glob

next_id=1
old_to_new_id = dict()
old_to_new_id[0] = 0 # The distinguished vertex always has id 0.

def get_id(old_id):
    global next_id
    global old_to_new_id
    if not old_id in old_to_new_id:
        old_to_new_id[old_id] = next_id
        next_id += 1
    return old_to_new_id[old_id]

def convert_to_consecutive(oldfile, newfile):
    """
    Converts the graph in oldfile to consecutive ids and saves it to newfile.
    """
    global next_id
    global old_to_new_id

    f = open(oldfile, "r")
    lines = map(lambda line: line.split(), f.read().splitlines())
    f.close()

    next_id=1
    old_to_new_id = dict()
    old_to_new_id[0] = 0 # The distinguished vertex always has id 0.

    f = open(newfile, "w")
    for line in lines:
        from_id = get_id(line[0])
        to_id = get_id(line[1])
        w = line[2]
        f.write("{} {} {}\n".format(from_id, to_id, w))

    f.close()


if __name__ == "__main__":

    if len(sys.argv) != 3:
        print "usage: {} old_dataset_folder new_dataset_folder".format(sys.argv[0].split("/")[-1])
        sys.exit(1)

    old_folder_path = sys.argv[1]
    new_folder_path = sys.argv[2]

    os.system("mkdir -p \"{}\"".format(new_folder_path))
    for index, oldfile in enumerate(glob.glob("{}/*.txt".format(old_folder_path))):
        newfile = "{}/{}".format(new_folder_path, oldfile.split("/")[-1])
        print "[{}] oldfile: {}".format(index, oldfile)
        print "[{}] newfile: {}".format(index, newfile)
        convert_to_consecutive(oldfile, newfile)

    print "Saved new consecutive graph dataset to \"{}\"".format(new_folder_path)
