"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""



#!/usr/bin/python

"""
Creates an actor-movie-neighbor adjacency list from the SQL version of the IMDb.

Each line in the created adjlist is as follows:
    my_id you_id1 movie_id1 you_id2 movie_id2 ...

where the movie_id is the id of the movie my_id and you_id appeared together in.

The real names of persons and movies can be gotten by looking up the ID in the SQL version of the IMDb,
as created by IMDbPY.

Written by Tommi Kerola - 2013-04-16
"""

# Selects all neighbors for a specific person.
query_select_nbrs = "SELECT p_name.id AS me, n_ci.person_id AS you, p_ci.movie_id AS movie FROM name AS p_name JOIN cast_info AS p_ci ON p_ci.person_id = p_name.id JOIN title as p_mt ON p_mt.id = p_ci.movie_id JOIN cast_info AS n_ci ON n_ci.movie_id = p_ci.movie_id WHERE p_name.id = ? AND p_mt.production_year BETWEEN 1994 AND 2003 AND p_name.id != n_ci.person_id AND p_mt.kind_id = 1;"

# The maximum id of the imdb.name SQL table. The ids start at 1.
max_name_id = 2666147

def sql_exec(curs, q,p):
    st = time.time()
    curs.execute(q, p)
    print "query time: {}".format(time.time() - st)

def write_neighborhood(outfile, curs, actor_id):
    global query_select_nbrs

    # Get neighbors from SQL
    curs.execute(query_select_nbrs, [actor_id])
    # Write line of adjacency list
    outfile.write("{} {}\n".format(actor_id, " ".join(map(lambda x: " ".join([str(x["you"]), str(x["movie"])] ), curs.fetchall()))))

if __name__ == "__main__":

    import oursql
    import time
    import numpy as np
    import sys

    if len(sys.argv) != 2:
        print "usage: {} outfilename".format(sys.argv[0])
        sys.exit(1)

    julia_id = 2436318
    start_year = 1994
    end_year = 2003
    min_actors_per_movie = 2

    conn = oursql.connect(db="imdb", user="root")
    curs = conn.cursor(oursql.DictCursor)

    st = time.time()

    outfile = open(sys.argv[1], "w")
    for actor_id in range(1, max_name_id+1):
        if actor_id % 1000 == 0:
            print "[{}] {}%: {} elapsed secs.".format(actor_id, 100.0*actor_id/float(max_name_id), time.time()-st)
        write_neighborhood(outfile, curs, actor_id)
    outfile.close()

