"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""


#!/usr/bin/python
# -*- coding: utf-8 -*- 

"""
Parses the SQL version of the IMDb to create a synthetically ambiguous
relational graph.

NOTE: Oudated as of 130423. /Tommi
"""

import sys
import itertools
from collections import defaultdict

next_index = 0
rfIdToIndex = dict()

# The maximum id of the imdb.name SQL table. The ids start at 1.
max_name_id = 2666147

def get_index(rf_id):
    global next_index
    if not rf_id in rfIdToIndex:
        rfIdToIndex[rf_id] = next_index
        next_index += 1
    return rfIdToIndex[rf_id]

def get_pairs(vals):
    args = [iter(vals)] * 2
    return list(itertools.izip_longest(*args))

def extract_neighborhood(k, adjlistfilename, outfilename, actor_id):
    rf_ids_to_process = []
    rf_ids_to_process_next_round = [actor_id]
    rf_ids_chosen = []

    adjlistfile = open(adjlistfilename, "r")
    for line in adjlistfile:
        vals = line.split()
        pairs = vals[1:]
        if len(pairs) == 0:
            continue
        actor_id = get_index(int(vals[0]))
        nbrs=defaultdict(int)
        for to_id,mid in pairs:
            nbrs[to_id] += 1    # Co-occurence in a movie!
        for to_id,w in nbrs.items():
            if i<k or to_rf_id in rf_ids_chosen:
                to_index = get_index(to_rf_id)
                nbrs+=1
                if from_index < to_index: # Make sure we do not draw double edges
                    rf_ids_to_process_next_round.append(to_rf_id)
                    outfile.write("{} {} {}\n".format(from_index, to_index, w))
        print "{}: found {} nbrs".format(line_id, nbrs)

    outfile = open(outfile, "w")
    for i in range(k+1):
        rf_ids_to_process = rf_ids_to_process_next_round
        rf_ids_chosen.extend(rf_ids_to_process)
        rf_ids_to_process_next_round = []
        print "k={}".format(i)
        f = open(adjlistfilename, "r")
        for line in f:
            line_id = line.split()[0]
            if line_id in rf_ids_to_process:
                rf_ids_to_process.remove(line_id)
                from_index = get_index(line_id)
                vals = line.split()[1:] # Exclude the from_id
                pairs = get_pairs(vals)
                nbrs=0
                for to_rf_id,w in pairs:
                    if i<k or to_rf_id in rf_ids_chosen:
                        to_index = get_index(to_rf_id)
                        nbrs+=1
                        if from_index < to_index: # Make sure we do not draw double edges
                            rf_ids_to_process_next_round.append(to_rf_id)
                            outfile.write("{} {} {}\n".format(from_index, to_index, w))
                print "{}: found {} nbrs".format(line_id, nbrs)
        f.close()
    outfile.close()
    print "Wrote {} graph to \"{}\"".format(rf_id, outfilename)

def get_label(actor_id):
    import oursql
    # Get actor name
    query = "SELECT "


if __name__ == "__main__":
    if len(sys.argv) != 3:
        print __doc__
        print "usage: {} imdb_adjlist.txt outputfolder".format(sys.argv[0])
        sys.exit(1)

    adjlistfilename = sys.argv[1]
    outfolder = sys.argv[2]
    k=1

    adjlistfile = open(adjlistfilename, "r")
    for line in adjlistfile:
        vals = line.split()
        pairs = vals[1:]
        if len(pairs) == 0:
            continue
        actor_id = get_index(int(vals[0]))
        label, actor_ids = get_label(actor_id)
        extract_neighborhood(k, adjlistfilename, "{}/{}.{}.txt".format(outfolder, actor_id, label), actor_ids)
    adjlistfile.close()

