"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""

#!/usr/bin/python

"""
Creates a qsub script file for doing a grid search on Pegasos parameters.
"""

import sys
import os
import datetime

def get_date_str():
    d=datetime.datetime.now()
    return "{0:02}{1:02}{2:02}_{3:02}{4:02}{5:02}".format(d.year, d.month, d.day, d.hour, d.minute, d.second)

if __name__ == "__main__":

    if len(sys.argv) != 3:
        print "usage: {} halfkernellist.txt num_threads".format(sys.argv[0])
        sys.exit(1)

    halfkernellist_filename = sys.argv[1]
    num_threads = int(sys.argv[2])

    script_name="job_{}.{}.sh".format(halfkernellist_filename.split("/")[-1], get_date_str())

    os.system("split -d -n {} {} {}.".format(num_threads, halfkernellist_filename, halfkernellist_filename))

    # Run the Pegasos tests in parallel for each file.
    cmd_str="ls {}.* | xargs -P {} -I myfile python ../scripts/test_pegasos_params.py myfile".format(halfkernellist_filename, num_threads)

    out_str= "#!/bin/bash\n#$ -V\n#$ -cwd\n"
    out_str+="#$ -pe pe_mpi {}\n".format(num_threads)
    out_str+="echo {}\n".format(script_name)
    out_str+="echo \"{}\"\n".format(cmd_str)
    out_str+=cmd_str+"\n"

    f=open(script_name, "w")
    f.write(out_str)
    f.close()

    print "Wrote qsub script file to \"{}\"".format(script_name)
