"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""


#!/usr/bin/python

"""
Creates a script file usable for the Grid Engine qsub command to use.

Options:
- scriptfile_prefix
- num_threads          (MPI threads))
- num_cpus             (OpenMP threads)
- no_cv                (true or false)
- trainfile
    - directed_edges    (true or false)
    - edge_transform    (log, inv, none)
- normalize             (true or false)
- kernel            (sp or rw)
    - rsp_binning   (lin or log or revlog)
    - rsp_num_bins
    - dp_lambda
- name prediction
    - namesfile
    - use_names
- program type
    - train_svm or predictor
"""

import sys
import datetime

if len(sys.argv) == 15 and sys.argv[9] == "rsp":
    prefix=sys.argv[1]
    num_threads=int(sys.argv[2])
    num_cpus=int(sys.argv[3])
    no_cv=sys.argv[4]
    trainfile=sys.argv[5]
    directed_edges=sys.argv[6]
    edge_transform=sys.argv[7]
    normalize=sys.argv[8]
    kernel=sys.argv[9]
    rsp_binning=sys.argv[10]
    rsp_num_bins=sys.argv[11]
    namesfile = sys.argv[12]
    use_names = sys.argv[13]
    program_type = sys.argv[14]    

    extra_args="--directed {} ".format(directed_edges)
    if edge_transform == "inv":
        extra_args+="--invedges true "
    elif edge_transform == "log":
        extra_args+="--logedges true "
    extra_args+="--rsp_binning {} --rsp_num_bins {} --normalize_half_kernels {} --no_cv {} --use_names {}".format(rsp_binning, rsp_num_bins, normalize, no_cv, use_names)

elif len(sys.argv) == 15 and sys.argv[9] == "dp":
    prefix=sys.argv[1]
    num_threads=int(sys.argv[2])
    num_cpus=int(sys.argv[3])
    no_cv=sys.argv[4]
    trainfile=sys.argv[5]
    directed_edges=sys.argv[6]
    edge_transform=sys.argv[7]
    normalize=sys.argv[8]
    kernel=sys.argv[9]
    dp_lambda=sys.argv[10]
    dp_use_distinguished=sys.argv[11]
    namesfile = sys.argv[12]
    use_names = sys.argv[13]
    program_type = sys.argv[14]

    extra_args="--directed {} ".format(directed_edges)
    if edge_transform == "inv":
        extra_args+="--invedges true "
    elif edge_transform == "log":
        extra_args+="--logedges true "
    extra_args+="--dp_lambda {} --normalize_half_kernels {} --dp_distinguished_only {} --no_cv {} --use_names {}".format(dp_lambda, normalize, dp_use_distinguished, no_cv, use_names)
else:
    print "usage: {} prefix num_threads ncpus no_cv trainfile directed_edges edge_transform normalize_half_kernel rsp rsp_binning rsp_num_bins namesfile use_names program_type".format(sys.argv[0])
    print "usage: {} prefix num_threads ncpus no_cv trainfile directed_edges edge_transform normalize_half_kernel dp dp_lambda dp_use_distinguished namesfile use_names program_type".format(sys.argv[0])
    sys.exit(1)

def get_date_str():
    d=datetime.datetime.now()
    return "{0:02}{1:02}{2:02}_{3:02}{4:02}{5:02}".format(d.year, d.month, d.day, d.hour, d.minute, d.second)

script_name="job_{}_{}.sh".format("_".join(sys.argv[1:12] + sys.argv[13:]), get_date_str())
halfkernels_name="halfkernels_{}_{}.txt".format("_".join(sys.argv[1:12] + sys.argv[13:]), get_date_str())

if program_type == "train_svm":
        cmd_str="/usr/opt/openmpi-1.4.5/bin/mpirun -np {0} --hostfile hostfile ../build/src/train_svm ../build/{1} {2} --ncpus {3} --ho {4} {5} --namesfile {6}".format(num_threads, trainfile, kernel, num_cpus, halfkernels_name, extra_args, namesfile)

elif program_type == "predictor":
        cmd_str="/usr/opt/openmpi-1.4.5/bin/mpirun -np {0} --hostfile hostfile ../build/src/predictor ../build/{1} {2} --ncpus {3} --ho {4} {5} --namesfile {6}".format(num_threads, trainfile, kernel, num_cpus, halfkernels_name, extra_args, namesfile)

else: 
    print "usage: {} prefix num_threads ncpus no_cv trainfile directed_edges edge_transform normalize_half_kernel rsp rsp_binning rsp_num_bins use_names program_type".format(sys.argv[0])
    print "usage: {} prefix num_threads ncpus no_cv trainfile directed_edges edge_transform normalize_half_kernel dp dp_lambda dp_use_distinguished use_names program_type".format(sys.argv[0])
out_str= "#!/bin/bash\n#$ -V\n#$ -cwd\n"
out_str+="#$ -pe pe_mpi {}\n".format(num_threads*num_cpus)
out_str+="echo {}\n".format(script_name)
out_str+="echo \"{}\"\n".format(cmd_str)
out_str+=cmd_str+"\n"

f=open(script_name, "w")
f.write(out_str)
f.close()

print "Wrote qsub script file to \"{}\"".format(script_name)

