"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""


#!/usr/bin/python

"""
Creates a qsub script file for doing a grid search for semi-sup. learning.
"""

import sys
import os
import datetime

def get_date_str():
    d=datetime.datetime.now()
    return "{0:02}{1:02}{2:02}_{3:02}{4:02}{5:02}".format(d.year, d.month, d.day, d.hour, d.minute, d.second)

if __name__ == "__main__":

    if len(sys.argv) != 4:
        print "usage: {} labeled_halfkernels.txt unlabeled_halfkernels.txt unlabeled_fraction".format(sys.argv[0])
        sys.exit(1)

    labeled_halfkernels_filename = sys.argv[1]
    unlabeled_halfkernels_filename = sys.argv[2]
    unlabeled_fraction = float(sys.argv[3])

    script_name="job_{}_{}_frac_{}.{}.sh".format(labeled_halfkernels_filename.split("/")[-1], unlabeled_halfkernels_filename.split("/")[-1], unlabeled_fraction, get_date_str())

    # Run the Pegasos tests in parallel for each file.
    cmd_str="python ../scripts/do_semi_supervised_training.py {} {} {}".format(labeled_halfkernels_filename, unlabeled_halfkernels_filename, unlabeled_fraction)

    out_str= "#!/bin/bash\n#$ -V\n#$ -cwd\n"
    out_str+="#$ -pe pe_mpi {}\n".format(1)
    out_str+="echo {}\n".format(script_name)
    out_str+="echo \"{}\"\n".format(cmd_str)
    out_str+=cmd_str+"\n"

    f=open(script_name, "w")
    f.write(out_str)
    f.close()

    print "Wrote qsub script file to \"{}\"".format(script_name)
