"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""

#!/usr/bin/python

import sys
import os
import itertools
import random

num_folds = 10
svmargs = "-t 4 -c 1000"

if len(sys.argv) < 2:
    print "usage: cv_libsvm.py svmfile"
    sys.exit(1)

svmfile = sys.argv[1]

if len(sys.argv) == 3:
    svmargs = sys.argv[2]

f = open(svmfile, "r")
lines_raw = f.read().splitlines()
f.close()

# Make sure the lines are shuffled before training the SVM.
random.shuffle(lines_raw)

def create_folds(a, k):
    return [[a[len(a)*n//k+i] for i in range(len(a)/k)] for n in range(k)]

def get_test_acc(mode, val_lines):
    print "{}: ".format(mode),
    sys.stdout.flush()
    os.system("svm-predict {0}.{1} {0}.model {0}.output".format(svmfile, mode))

    f = open("{}.output".format(svmfile), "r")
    result_lines = map(int, f.read().splitlines())
    f.close()

    correct_lines = map(lambda x: int(x.split(" ")[0]), val_lines)

    num_correct = map(lambda x: 1 if x[0] == x[1] else 0, zip(result_lines, correct_lines))
    acc = sum(num_correct)/float(len(num_correct))

    return acc

lines = create_folds(lines_raw, num_folds)

train_avg_acc = 0

avg_acc = 0
min_acc =  sys.maxint
max_acc = -sys.maxint

for i in range(num_folds):
    val_lines = lines[i]
    train_lines = lines[0:i] + lines[(i+1):num_folds]
    # flatten
    train_lines = list(itertools.chain.from_iterable(train_lines))

    f = open("{}.test".format(svmfile), "w")
    f.write("\n".join(val_lines))
    f.close()
    f = open("{}.train".format(svmfile), "w")
    f.write("\n".join(train_lines))
    f.close()
    
    print "\n{} of {}".format(i+1,num_folds)
    os.system("svm-train {1} {0}.train {0}.model".format(svmfile, svmargs))

    train_acc = get_test_acc("train", train_lines)
    train_avg_acc += train_acc

    test_acc = get_test_acc("test", val_lines)
    avg_acc += test_acc
    min_acc = min(min_acc, test_acc)
    max_acc = max(max_acc, test_acc)

train_avg_acc /= float(num_folds)
avg_acc /= float(num_folds)

print ""
print "[train] avg_acc: {}".format(train_avg_acc)
print " [test] avg_acc: {}".format(avg_acc)
print " [test] min_acc: {}".format(min_acc)
print " [test] max_acc: {}".format(max_acc)
print "svmargs: {}".format(svmargs)
    


