"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""


#!/usr/bin/python

"""
Turns RF edgelist
1,2,3.8
2,4,5.2

into adj_list
1 2 3.8
2 1 3.8 4 5.2
4 2 5.2

Format is <id> <to1> <w1> <to2> <w2> ...
"""

import sys
from collections import defaultdict

if len(sys.argv) != 3:
    print "usage: {} edgelist.txt adjlist.txt".format(sys.argv[0])
    sys.exit(1)

def line_to_vals(l):
    a,b,c=l.split(",")
    return (a,b,c)

edgelist_filename = sys.argv[1]
adjlist_filename = sys.argv[2]

print "Reading {}".format(edgelist_filename)
lines = map(line_to_vals, open(edgelist_filename, "r").read().splitlines())

adjlist = defaultdict(set)

print "Creating adj. list."
i = 1
numlines = len(lines)
for a,b,c in lines:
    if i % 10000 == 0:
        print "{}%".format(100.0*i/float(numlines))
    adjlist[a] |= set([(b,c)])
    adjlist[b] |= set([(a,c)])
    i+=1

print "Writing adj. list to {}".format(adjlist_filename)
f = open(adjlist_filename, "w")
f.write("\n".join([" ".join(map(str, [a] + [b for t in list(s) for b in t])) for a,s in adjlist.items()]))
print "Done."

