"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""


#!/usr/bin/python

"""
Extracts the k-neighborhood of a node with the specified id.
"""

import sys
import itertools
import time

next_index = 0
rfIdToIndex = dict()

def get_index(rf_id):
    global next_index
    global rfIdToIndex
    if not rf_id in rfIdToIndex:
        rfIdToIndex[rf_id] = next_index
        next_index += 1
    return rfIdToIndex[rf_id]

def get_pairs(vals):
    args = [iter(vals)] * 2
    return list(itertools.izip_longest(*args))

def extract_neighborhood(start_rf_id, k, adjlistfilename, outfilename):
    import eta

    rf_ids_to_process = set()
    rf_ids_to_process_next_round = set([0])
    rf_ids_chosen = set()
    rf_ids_visited = set([0])

    tot_timer = eta.Timer()
    tot_timer.start()

    sys.stdout.write("Creating cache \n")
    sys.stdout.flush()
    rf_id_to_pairs = dict()
    f = open(adjlistfilename, "r")
    lines = map(lambda line: line.split(), f.read().splitlines())
    f.close()
    print "Converting to consecutive ids"
    new_id_to_lines_index = dict()
    old_to_new_id = dict()
    old_to_new_id[start_rf_id] = 0
    next_id = 1
    for index,line in enumerate(lines):
        rf_id = line[0]
        if not rf_id in old_to_new_id:
            old_to_new_id[rf_id] = next_id
            next_id+=1
        new_id_to_lines_index[old_to_new_id[rf_id]] = index
    num_ids=len(old_to_new_id)
    print "num_ids:",num_ids
    max_num_cols = reduce(lambda a,b: max(a,len(b)), lines, 0)
    print "max_num_cols:",max_num_cols
    print "({:.2f} secs)".format(tot_timer.get_elapsed())

    outfile = open(outfilename, "w")
    for i in range(k+1):
        rf_ids_to_process = rf_ids_to_process_next_round
        rf_ids_chosen.update(rf_ids_to_process)
        rf_ids_to_process_next_round = set()
        sys.stdout.write("k={} ".format(i))
        sys.stdout.flush()

        k_timer = eta.Timer(len(rf_ids_to_process))
        k_timer.start()
        eta_timer = eta.Timer()
        eta_timer.start()

        # Assume each line has the format: rf_id to_id1 w1 to_id2 w2 ...
        print "num_rf_ids_to_process: {}".format(len(rf_ids_to_process))
        for index,rf_id in enumerate(rf_ids_to_process): # If we find the rf_id we were searching for
            if index % 100 == 0 and index != 0:
                sys.stdout.write(".")
                if eta_timer.get_elapsed() > 60:
                    sys.stdout.write(" ({})".format(k_timer.format(k_timer.get_eta(index))))
                    eta_timer.start()
                sys.stdout.flush()

            try:
                lines_index = new_id_to_lines_index[rf_id]
            except:
                print "Not found"
                continue
            line = lines[lines_index]
            nbr_ids = line[1::2]
            nbr_weights = line[2::2]
            for nbr_index in xrange(len(nbr_ids)):
                try:
                    to_rf_id = old_to_new_id[nbr_ids[nbr_index]]
                except:
                    continue
                to_weight = nbr_weights[nbr_index]
                if i>=k and not to_rf_id in rf_ids_chosen:
                    continue

                if rf_id < to_rf_id: # Make sure we do not draw double edges
                    outfile.write("{} {} {}\n".format(rf_id, to_rf_id, to_weight))
                if not to_rf_id in rf_ids_visited:
                    rf_ids_visited.add(to_rf_id)
                    rf_ids_to_process_next_round.add(to_rf_id) # Previously processed ids have lower id, and do not process each id more than once.
            #print "{}: found {} nbrs".format(rf_id, nbrs)
        print "({:.2f} secs)".format(k_timer.get_elapsed())
    outfile.close()
    print "Wrote {} graph to \"{}\" ({:.2f} secs total)".format(start_rf_id, outfilename, tot_timer.get_elapsed())

if __name__ == "__main__":
    if len(sys.argv) != 5:
        print __doc__
        print "usage: {} nodeId k adjlistfilename outputgraph".format(sys.argv[0])
        sys.exit(1)

    rf_id = sys.argv[1]
    k = int(sys.argv[2])
    adjlistfilename = sys.argv[3]
    outfilename = sys.argv[4]
    extract_neighborhood(rf_id, k, adjlistfilename, outfilename)
