"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""

#!/usr/bin/python

"""
Removes self-edges from adjlist files.
These can occur when creating ambiguous actors if a movie contains several actors with the same last name.
"""

import sys

def remove_self_edges(inputfilename, outputfilename):
    """
    Removes self-edges and the adjacency list file and outputs a new correct adjlist file.

    Input: imdb_amb_adjlist_malin.txt
    Output: imdb_amb_adjlist_malin.txt.corrected
    """

    f = open(inputfilename, "r")
    lines = f.read().splitlines()
    f.close()
    num_lines = len(lines)
    totcount = 0
    poscount = 0
    negcount = 0

    outfile = open(outputfilename, "w")

    print "Stage 1: Check number of actors with self-edges."
    for index,line in enumerate(lines):
        vals = line.split()
        actor_id = vals[0]
        num_ents = int(vals[1])
        nbrs = vals[2:][::2] # The actors this actor has starred with.
        if actor_id in nbrs:
            #print "Warning: Actor {} has self-edge!".format(actor_id)
            totcount += 1
            if num_ents > 1:
                poscount += 1
            else:
                negcount += 1

            pairs = []
            for i in range(len(nbrs)):
                a = vals[2:][i*2]
                b = vals[2:][i*2+1]
                if actor_id != a:
                    pairs.append("{} {}".format(a,b))
            if len(pairs) > 0:
                outfile.write("{} {} ".format(actor_id, num_ents))
                outfile.write(" ".join(pairs))
                outfile.write("\n")

        else:
            # Print line as usual, it was correct from the beginning.
            outfile.write(line)
            outfile.write("\n")

        if index % 1000 == 0:
            print "[{}] {:.2f}%".format(actor_id, 100*index/float(num_lines))

    outfile.close()

    print "Num actors with self-edges: {} ({:.2f}%)".format(totcount, 100.0 * totcount / float(num_lines))
    print "of which +1: {:.2f}%".format(100.0 * poscount / float(totcount))
    print "         -1: {:.2f}%".format(100.0 * negcount / float(totcount))

if __name__ == "__main__":

    if len(sys.argv) != 3:
        print "usage: {} imdb_amblist.txt outfilename".format(sys.argv[0])
        sys.exit(1)

    inputfilename = sys.argv[1]
    outputfilename = sys.argv[2]

    remove_self_edges(inputfilename, outputfilename)
