"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""


#!/usr/bin/python
"""
==================================
          - plot.py -
==================================
Convenience script for plotting basic x,y values.
Each input file should contain lines with x[:space:]y.
Previous axis ranges are stores in the file .plotaxis,
which will be automatically updated when using the GUI.

Example:
$ cat file.txt
1 23
2 12
3 10
4 4
5 6

$ plot.py file.txt
"""

import sys
import matplotlib.pyplot as plt
import os

if len(sys.argv) < 2:
    print "usage: plot.py train_value.txt [test_value.txt ...]"
    print __doc__
    sys.exit(1)

fig = plt.figure()

for i in range(len(sys.argv)-1):
    filename = sys.argv[i+1]
    f = open(filename)
    lines = f.read().splitlines()
    f.close()

    x = map(float, [line.split()[0] for line in lines])
    y = map(float, [line.split()[1] for line in lines])

    ax = fig.add_subplot(111)
    ax.set_xlabel("t")
    ax.set_ylabel("value")
    ax.plot(x,y, label=filename)

if os.path.isfile(".plotaxis"):
    try:
        f = open(".plotaxis", "r")
        xmin,xmax,ymin,ymax = map(float, f.read().split())
        f.close()
        ax.set_xlim(xmin,xmax)
        ax.set_ylim(ymin,ymax)
        print "Set axis =", ax.axis()
    except:
        pass

ax.legend()
plt.show()

print "New axis =", ax.axis()
f = open(".plotaxis", "w")
f.write(" ".join(map(str, ax.axis())))
f.close()
print "Wrote axis to file \".plotaxis\""
