"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""

#!/bin/bash
# @author Tommi Kerola
# @date   2013-02-03
# Runs a single test between the GL and FW implementations on a specified graph.

GRAPHLAB_OR_GRAPHCHI="graphchi"
DIFF_CMD="diff -y --suppress-common-lines result.gl result.fw"
FW_CMD="build/src/floyd_warshall"
if [ "$GRAPHLAB_OR_GRAPHCHI" == "graphlab" ]; then
    echo "GraphLab mode."
    GL_CMD="./run.sh --format snap --directed 1 --graph"
else
    echo "GraphChi mode."
    GL_CMD="./run.sh --format snap filetype edgelist --directed true --niters 1000 --graph"
fi

if [ $# -ne 1 ]; then
    echo "usage: run_single_test.sh graph"
    exit 65
fi

GRAPHNAME="$1"

# Make sure we don't have an empty graph name (this will mess up the cleanup part otherwise).
if [ -z "$GRAPHNAME" ]; then
    echo -e "Graph name cannot be empty." 
    exit 65
fi

# Make sure we don't have a graph name with only spaces (this will mess up the cleanup part otherwise).
if [[ $GRAPHNAME =~ ^\ +$ ]]; then
  echo -e "Graph name cannot be empty." 
  exit 65
fi

function cleanup {
    # Remove previous files generated by GraphChi
    rm -rf ${GRAPHNAME}.*
    rm -rf ${GRAPHNAME}_*
}

# Remove previous files generated by GraphChi
cleanup

$GL_CMD $GRAPHNAME #&> /dev/null         # creates result_1_of_1
sort result_1_of_1 -n > result.gl        # make sure that the GL result is sorted, save it in result.gl
$FW_CMD $GRAPHNAME $GRAPHLAB_OR_GRAPHCHI #&> /dev/null        # creates result.fw
$DIFF_CMD                                # compares result.gl and result.fw

# Did the files differ?
if [[ "$?" != "0" ]]; then
    echo -e "\nGraphLab (result.gl)\t\t\t\t\t\tFloyd-Warshall(result.fw)"
    echo -e "\nFiles differ!"
    echo -e "Graph: $GRAPHNAME"
    echo -e "Results: result.{gl,fw}"
    cleanup
    exit
fi

echo -e "\nTest completed successfully."
cleanup
