"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""


#!/bin/bash
# @author Tommi Kerola
# @date   2013-02-01
# Generates lots of random graphs and checks that the parallel GL and serial FW algorithm get the same result.

NUM_TESTS=1000
N=10 # num nodes
E=10 # num edges

GRAPHLAB_OR_GRAPHCHI=graphlab
LOG_FILE="result.log"
DIFF_CMD="diff -y --suppress-common-lines result.gl result.fw"
FW_CMD="build/src/floyd_warshall"
if [ "$GRAPHLAB_OR_GRAPHCHI" == "graphlab" ]; then
    echo "GraphLab mode."
    GL_CMD="./run.sh --format snap --directed 1 --graph"
else
    echo "GraphChi mode."
    GL_CMD="./run.sh --format snap filetype edgelist --directed true --niters 1000 --graph"
fi

if [ $# -eq 1 ]; then
    NUM_TESTS=$1
elif [ $# -eq 3 ]; then
    NUM_TESTS=$1
    N=$2
    E=$3
fi

function cleanup {
    # Remove previous files generated by GraphChi
    rm -rf test_graph.*
    rm -rf test_graph_*
}

rm -f "$LOG_FILE"
echo "Running $NUM_TESTS tests with random graphs G=(V=$N,E=$E)."

for i in `seq 1 $NUM_TESTS`; do
    python genGraph.py $N $E > test_graph

    # Remove previous files generated by GraphChi
    cleanup

    $GL_CMD test_graph &>> "$LOG_FILE"          # creates result_1_of_1
    sort result_1_of_1 -n > result.gl          # make sure that the GL result is sorted, save it in result.gl
    $FW_CMD test_graph $GRAPHLAB_OR_GRAPHCHI &>> "$LOG_FILE"        # creates result.fw
    $DIFF_CMD &>> "$LOG_FILE"                   # compares result.gl and result.fw

    # Did the files differ?
    if [[ "$?" != "0" ]]; then
        $DIFF_CMD
        echo -e "\nGraphLab (result.gl)\t\t\t\t\t\tFloyd-Warshall(result.fw)"
        echo -e "\nFiles differ! Ran ${i} tests."
        echo -e "Graph: test_graph"
        echo -e "Results: result.{gl,fw}"
        echo "See log file for details: $LOG_FILE"
        cleanup
        exit
    else
        if [[ $((i%50)) == "0" && $i != 0 ]]; then
            echo ". ${i}"
        else
            echo -n "."
        fi
    fi
done

echo -e "\nAll $NUM_TESTS tests completed successfully."
cleanup
