"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""

#!/usr/bin/python

"""
Selects 1000 actors to use in the training set.
"""

import sys
import random

if __name__ == "__main__":

    if len(sys.argv) < 3 or len(sys.argv) > 4:
        print "usage: {} imdb_amb_adjlist_kernel.txt num_of_each_class [banned_nodes.txt]".format(sys.argv[0])
        sys.exit(1)

    inputfilename = sys.argv[1]
    num_of_each_class = int(sys.argv[2])
    banlist=set()
    if len(sys.argv) == 4:
        bannedlist_filename = sys.argv[3]
        #print "Using banlist: \"{}\"".format(bannedlist_filename)
        f = open(bannedlist_filename, "r")
        for line in f:
            banlist.add(line)
        f.close()

    f=open(inputfilename, "r")
    lines = f.read().splitlines()
    f.close()
    num_lines = len(lines)

    #print "Extracting {} pos. and neg. nodes. {} in total.".format(num_of_each_class, 2*num_of_each_class)
    num_pos = num_of_each_class
    num_neg = num_of_each_class

    ps=set()
    ns=set()

    # Limit min num. sources per actor.
    min_source_limit = 1
    # Limit max num. sources per actor.
    max_source_limit = 200
    # Limit max k-ambiguity per actor.
    max_k_amb_limit = 1000

    tries=0
    while not (len(ps) == num_pos and len(ns) == num_neg):
        tries+=1
        index = random.randint(0,num_lines-1)
        line = lines[index]
        vals = line.split()
        actor_id = vals[0]
        if actor_id in banlist:
            continue
        num_ents = int(vals[1])
        if num_ents > max_k_amb_limit:
            continue
        movies = set(vals[2:][1::2]) # The movies this actor has starred in.
        num_sources = len(movies)
        if num_sources < min_source_limit or num_sources > max_source_limit:
            continue

        if num_ents > 1:
            label = 1
            if len(ps) < num_pos:
                ps.add((actor_id, label))
        else:
            label = -1
            if len(ns) < num_neg:
                ns.add((actor_id, label))

        #if tries % 1000 == 0:
        #    print "tries: {} ps: {} ns: {}".format(tries, len(ps), len(ns))

    for (a,b) in ps:
        print "{} {}".format(a,b)
    for (a,b) in ns:
        print "{} {}".format(a,b)
