"""
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
"""



#!/usr/bin/python

"""
Tests various Pegasos params on the specified halfkernels.
"""

import sys
import os

if len(sys.argv) != 2:
    print "usage: {} halfkernel_files_list".format(sys.argv[0])
    sys.exit(1)

lambdas=[10**(-i) for i in range(1, 10)]
Ts=[10**i for i in range(0,6)]
print "lambdas: {}".format(lambdas)
print "Ts: {}".format(Ts)

f=open(sys.argv[1], "r")
halfkernel_files = f.read().splitlines()
f.close()
print "Found {} halfkernel files.".format(len(halfkernel_files))

tot = len(halfkernel_files) * len(lambdas) * len(Ts)
c=0;
print "Running {} Pegasos param tests.".format(tot)
for halfkernel_file in halfkernel_files:
    for lamb in lambdas:
        for T in Ts:
            outfilename = halfkernel_file.split("/")[-1] + ".pegasos{}_{}_.out".format(lamb, T)
            os.system("../build/src/train_svm trainfile_rf1 rsp --i {} --pegasos_lambda {} --pegasos_iter {} > {}".format(halfkernel_file, lamb, T, outfilename))
            c+=1
        print "{}%".format(100*float(c)/tot)

