"""
 * @file
 *
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
 * @section DESCRIPTION
"""


import sys
import numpy as np
import time
import math
import urllib
import json
from collections import defaultdict
import unicodedata

URL = 'http://api.recordedfuture.com/query?%s'
TOKEN = '9EbBBB' #Linus Token
MAX_INSTANCES = 100

def main():

    MAX_NEIGHBOURS = 10
    START_TIME = "1990-01-01"
    STOP_TIME = "2014-01-01"


    #The "output_path" should be a folder not a file like this "output/" not "output"
    if len(sys.argv) != 3:
	print "usage: main entities_path output_path"
	sys.exit(1)


    entities = []    
    file = open(sys.argv[1], "r")

    while True:
	line = file.readline()
	if line == "":
	    break
	entities.append(line.rstrip())

    file.close()
    print "list of entities" + str(entities)
    
    names = []
    entity_num = 1


    start_time = time.time()
    query_time = 0
    skip_counter = 0

    

    for entity in entities:


	print "======================="
	print "====New Entity========="
	print "====" + str(entity) + "========="
	print "======================="
	#entities_to_sources = defaultdict(list) #dictionary with a list of all sources for an entity
	sources_to_entities = dict() #dictionary with a list of all soruces for an entity
	sources = []	
	neighbours = []

	#Query to get the name

	query = {
		    "entity":
		    {
			"limit": 1,
			"id":entity
		    },
		    "token": TOKEN
		}
		    

 	temp_time = time.time()
	query_response = submitQuery(query)
	query_time += time.time() - temp_time	

	#print query_response
    	if query_response["status"] == "failure":
	    raise Exception("Query didn't work\n {}".format(query_response))   

	names.append(query_response["entity_details"][entity]["name"])

	#Query specific to the entity we are about to create the neighborhood on
	query = {
		    "instance":
		    {
			"attributes":[{"entity":{"id":entity}}],"limit": MAX_INSTANCES
		    },
		    "output":{"count":{"axis":["document"],"values":[]}},
		    "token": TOKEN
		} 



	#print type(query)
	#print query
	temp_time = time.time()
	query_response = submitQuery(query)
	query_time += time.time() - temp_time	

	#print query_response
    	if query_response["status"] == "failure":
	    raise Exception("Query didn't work\n {}".format(query_response))
    

	"""
	print(query_response)
	print "Ordered repsonse to query"
	for key in query_response:
	    print key
	    print query_response[key]
	    print "NEW KEY -------------------------"
	    print "---------------------------------"

	
	print "\n ENTITES query_response \n"
	print "******************************"
	for e in query_response["entities"]:
	    print e
	    print query_response["entities"][e]
	    print "NEW ENTITY************************"
	    print "**********************************"


	print "\n INSTANCES query_response \n"
	print "////////////////////////////////"
	for i in query_response["instances"]:
	    print i
	    #print query_response["instances"][i]
	    print "NEW INSTANCE/////////////////////////"
	    print "///////////////////////////////////"
	"""
    
	#collect all the documents in where the entity was mentioned so that we can query each
	#one and get the entities for each source
	for c in query_response["counts"]:
	    for c2 in c:
		sources.append(str(c2))
			
	#print "sources " + str(sources)
	
	for source in sources:

	    temp_time = time.time()
	    query_response = get_document(source)
	    query_time += time.time() - temp_time    	    
	    #print query_response


	    """
	    print(query_response)
	    print "Ordered repsonse to query"
	    for key in query_response:
		print key
		print query_response[key]
		print "NEW KEY -------------------------"
		print "---------------------------------"

	
	    print "\n ENTITES query_response \n"
	    print "******************************"
	    for e in query_response["entities"]:
		print e
		print query_response["entities"][e]
		print "NEW ENTITY************************"
		print "**********************************"


	    print "\n INSTANCES query_response \n"
	    print "////////////////////////////////"
	    for i in query_response["instances"]:
		print i
		#print query_response["instances"][i]
		print "NEW INSTANCE/////////////////////////"
		print "///////////////////////////////////"
	    """
	    
	    source_entities = []

	    for e in query_response["entities"]:		
		source_entities.append(str(e))
		#entities_to_sources[e].append(source)
	
	    	
	    #In case we got unlucky with the instances of the document and the entity was not included in it
	    if entity not in source_entities:
		source_entities.append(entity)

	    #print "source entities " + str(source_entities)
	    
	    if entity not in source_entities:
		raise Exception("The entity was not included in a document which was returned for the query of all its documents. Document {}. Entity {}".format(source, entity) ) 

	    sources_to_entities[source] = source_entities
	    #print "sources to entites " + str(sources_to_entities)
	    neighbours.extend(source_entities)

 
	neighbours = list(set(neighbours))
	#print "neighbours " + str(neighbours)
	neighbours.remove(entity) #the entity is not an neighbour of itself
	neighbours = neighbours[0:MAX_NEIGHBOURS]


	print "neighbours trimmed to MAX_NEIGHBOURS" + str(neighbours)

	#print "entites to sources " + str(entities_to_sources)
	#print "sources to entities" + str(sources_to_entities)


	#Query for all neighbours to complete the 1-neighbourhood
	for neighbour in neighbours:
		print "neighbour " +str(neighbour)

	    	query = {
		    "instance":
		    {
		    "attributes":[{"entity":{"id":neighbour}}],"limit": MAX_INSTANCES
		    },
		    "output":{"count":{"axis":["document"],"values":[]}},
		    "token": TOKEN
		} 


		sources = []	
		temp_time = time.time()
		query_response = submitQuery(query)
		query_time += time.time() - temp_time
    
    	    	for c in query_response["counts"]:
		    for c2 in c:
			sources.append(str(c2))
			
		#print "sources " + str(sources)

		#print "num of sources " + str(len(sources))
		for source in sources:

		    if source in sources_to_entities: #no need to query since we have the info already
			skip_counter+=1
			print "skipped queries " + str(skip_counter)
			continue

		    temp_time = time.time()
		    query_response = get_document(source)
		    query_time += time.time() - temp_time
		    source_entities = []

		    for e in query_response["entities"]:		
			source_entities.append(str(e))
			#entities_to_sources[e].append(source)


		    if neighbour not in source_entities:
			source_entities.append(neighbour)

		    #print "source entities " + str(source_entities)
		    sources_to_entities[source] = source_entities

	#for e in entities_to_sources:
	    #entities_to_sources[e] = list(set(entities_to_sources[e]))
	#print "entities to sources without duplicates " + str(entities_to_sources)
	#print "sources to entities " + str(sources_to_entities)


	edges = dict()
	relevant_nodes = neighbours[:]
	relevant_nodes.append(entity)
	#print "relevant nodes " + str(relevant_nodes)
	for rel_node in relevant_nodes:
	    edges[rel_node] = defaultdict(int)


	current_entities = []
        #create the edge list format for the neighbourhood
        for s in sources_to_entities:
	    current_entities = []
	    for e in sources_to_entities[s]:
		current_entities.append(e)
		
	    #print "Current entities " + str(current_entities)	    

	    for i in range(0,len(current_entities)):
		for j in range(i+1,len(current_entities)):
		    if (current_entities[i] in relevant_nodes) and (current_entities[j] in relevant_nodes):
			edges[ current_entities[i] ][ current_entities[j] ] += 1


	#print "edges " + str(edges)


	#Print neighbourhood to file.

	translate_to_int = dict()
	i = 1
	for node in edges:
	    translate_to_int[node] = i
	    i+=1	

	#print "translate_to_int " + str(translate_to_int)


	new_neighbourhood = sys.argv[2] + str(entity_num)
	file = open(new_neighbourhood, "w+")
	for node1 in edges:
	    for node2 in edges[node1]:
		file.write("{} {} {}\n".format(translate_to_int[node1], translate_to_int[node2], edges[node1][node2]))
	
	file.close()
	entity_num += 1



    new_names = sys.argv[2] + "names"
    file = open(new_names, "w+")
    for name in names:
	file.write("{}\n".format(name))
    file.close()

    total_time = time.time() - start_time
    print "total time: " + str(total_time)
    print "query time: " + str(query_time)
    print "time spent on other things than query: " + str(total_time - query_time) 


def get_instance(instance_id):

    q={
	"instance": {
	    "id":instance_id,
	    "limit":MAX_INSTANCES
	},
	"token":TOKEN
    }
    	
    #print (type (q))
    #print q
	
    d = submitQuery(q)
    """
    print "Full response to query"
    print(d)
    print "Ordered repsonse to query"
    for key in d:
	print key
    	print d[key]
	
    print "\n ENTITES \n"
    for e in d["entities"]:
	print "NEW ENTITY ============================"
    	print e
    	print d["entities"][e]
	
    print "\n INSTANCES \n"
    for i in d["instances"]:
    	print "NEW INSTANCE-----------------------"                
	print i
    """
	
    return d


def get_document(doc_id):

    q={
	"instance": {

	"document":{"id":doc_id}
	},
	"token":TOKEN
    }
    	
    #print (type (q))
    #print q
	
    d = submitQuery(q)
    """
    print "Full response to query"
    print(d)
    print "Ordered repsonse to query"
    for key in d:
	print key
    	print d[key]
	
    print "\n ENTITES \n"
    for e in d["entities"]:
	print "NEW ENTITY ============================"
    	print e
    	print d["entities"][e]
	
    print "\n INSTANCES \n"
    for i in d["instances"]:
    	print "NEW INSTANCE-----------------------"                
	print i
    """
	
    return d


def submitQuery(query_string):
    if isinstance(query_string,dict):
	try:
	    #print "QString b4 string " + str(query_string)
	    query_string = query2string(query_string)
	    #print (type (query_string))
	    #print "QString after string"
            #print query_string
	except Exception as e:
	    return "submitQuery: Error converting dict to string"
    try:
	#print "ULRL encoded string " + str(urllib.urlencode({"q":str(query_string)}))
        #print "Submitted QString " + str(URL % urllib.urlencode({"q":str(query_string)}))
	data = urllib.urlopen(URL % urllib.urlencode({"q":str(query_string)}))
	if type(data) != str:
	    # Note that data.read() can only be done once.
	    read_data = data.read()
	    return json.loads(read_data.decode('latin-1'))

	print "Return Value " + str(data)
	raise Exception("Query didn't return a good value ")
	return "Error";
    except Exception as e:
	print "Return Value " + str(data)
	raise Exception("Query didn't return a good value")

def query2string(query_dict):
    return '"'.join(str(query_dict).split("'"))

	
if __name__=="__main__":
    main()	

