"""
 * @file
 *
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
 * @section DESCRIPTION
"""


import sys
import numpy as np
import time
import math
import urllib
import json
from collections import defaultdict
import unicodedata

URL = 'http://api.recordedfuture.com/query?%s'
TOKEN = '9EbBBB' #Linus Token
MAX_INSTANCES = 1000

def main():

    MAX_NEIGHBOURS = 100


    #The "output_path" should be a folder not a file like this "output/" not "output"
    if len(sys.argv) != 3:
	print "usage: main entities_path output_path"
	sys.exit(1)


    entities = []    
    file = open(sys.argv[1], "r")

    while True:
	line = file.readline()
	if line == "":
	    break
	entities.append(line.rstrip())

    file.close()
    print "list of entities" + str(entities)
    
    names = []

    start_time = time.time()
    query_time = 0


    for entity in entities:

	#Query to get the name
	print entity
	query = {
		    "entity":
		    {
			"limit": 1,
			"id":entity
		    },
		    "token": TOKEN
		}
		    

 	temp_time = time.time()
	query_response = submitQuery(query)
	query_time += time.time() - temp_time	

	#print query_response
    	if query_response["status"] == "failure":
	    raise Exception("Query didn't work\n {}".format(query_response))   

	names.append(query_response["entity_details"][entity]["name"])

    new_names = sys.argv[2] + "names.txt"
    file = open(new_names, "w+")
    for name in names:
	file.write("{}\n".format( name.encode('utf8') )) #This encoding destroys certain names
    file.close()

    total_time = time.time() - start_time
    print "total time: " + str(total_time)
    print "query time: " + str(query_time)
    print "time spent on other things than query: " + str(total_time - query_time) 
    print "Extraction completed successfully"


def get_instance(instance_id):

    q={
	"instance": {
	    "id":instance_id,
	    "limit":MAX_INSTANCES
	},
	"token":TOKEN
    }
    	
    #print (type (q))
    #print q
	
    d = submitQuery(q)
    """
    print "Full response to query"
    print(d)
    print "Ordered repsonse to query"
    for key in d:
	print key
    	print d[key]
	
    print "\n ENTITES \n"
    for e in d["entities"]:
	print "NEW ENTITY ============================"
    	print e
    	print d["entities"][e]
	
    print "\n INSTANCES \n"
    for i in d["instances"]:
    	print "NEW INSTANCE-----------------------"                
	print i
    """
	
    return d


def get_document(doc_id):

    q={
	"instance": {

	"document":{"id":doc_id}
	},
	"token":TOKEN
    }
    	
    #print (type (q))
    #print q
	
    d = submitQuery(q)
    """
    print "Full response to query"
    print(d)
    print "Ordered repsonse to query"
    for key in d:
	print key
    	print d[key]
	
    print "\n ENTITES \n"
    for e in d["entities"]:
	print "NEW ENTITY ============================"
    	print e
    	print d["entities"][e]
	
    print "\n INSTANCES \n"
    for i in d["instances"]:
    	print "NEW INSTANCE-----------------------"                
	print i
    """
	
    return d


def submitQuery(query_string):
    if isinstance(query_string,dict):
	try:
	    #print "QString b4 string " + str(query_string)
	    query_string = query2string(query_string)
	    #print (type (query_string))
	    #print "QString after string"
            #print query_string
	except Exception as e:
	    return "submitQuery: Error converting dict to string"
    try:
	#print "ULRL encoded string " + str(urllib.urlencode({"q":str(query_string)}))
        #print "Submitted QString " + str(URL % urllib.urlencode({"q":str(query_string)}))
	data = urllib.urlopen(URL % urllib.urlencode({"q":str(query_string)}))
	if type(data) != str:
	    # Note that data.read() can only be done once.
	    read_data = data.read()
	    return json.loads(read_data.decode('latin-1'))

	print "Return Value " + str(data)
	print type(data)
	raise Exception("Query didn't return a good value ")
	return "Error";
    except Exception as e:
	print "Return Value " + str(data)
	print type(data)
	raise Exception("Query didn't return a good value")

def query2string(query_dict):
    return '"'.join(str(query_dict).split("'"))

	
if __name__=="__main__":
    main()	

