/**
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
 * @section DESCRIPTION
 *
 * Generic graph kernel.
 */
#ifndef GRAPH_KERNEL_HPP
#define GRAPH_KERNEL_HPP

#include "graphlabchi.hpp"

template<class VertexType, class EdgeType>
class GraphKernel {

    public:

    typedef graphlab::distributed_graph<VertexType, EdgeType> GraphType;

        /**
         * Constructs a new graph kernel for comparing the two graphs g1 and g2.
         */
        GraphKernel(GraphType& _g1, GraphType& _g2, graphlab::distributed_control& _dc, std::string _engine_type, graphlab::command_line_options& _clopts) : 
            g1(_g1), g2(_g2), dc(_dc), engine_type(_engine_type), clopts(_clopts) {}

        /**
         * Calculates the value of the kernel.
         */
        virtual double calculate() = 0;

    protected:
    GraphType& g1;
    GraphType& g2;
    graphlab::distributed_control& dc;
    graphlab::command_line_options& clopts;
    std::string engine_type;

};

#endif
