/**
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
 * @section DESCRIPTION
 *
 * GraphLab and GraphChi includes.
 */
#ifndef GRAPHLABCHI_HPP
#define GRAPHLABCHI_HPP

// Define whether to use GraphChi or GraphLab
#define USING_GRAPHCHI
#undef USING_GRAPHCHI

#ifdef USING_GRAPHCHI
#define MTLT_MAIN_DECLARATION int main(int argc, const char** argv)
#else
#define MTLT_MAIN_DECLARATION int main(int argc, char** argv)
#endif

// Enable MPI support.
#define HAS_MPI
//#define HAS_EIGEN
//#define HAS_TCMALLOC
//#define HAS_HADOOP

/**
 * \def LOG_NONE
 *   No info.
 * \def LOG_FATAL
 *   Used for fatal and probably irrecoverable conditions
 * \def LOG_ERROR
 *   Used for errors which are recoverable within the scope of the function
 * \def LOG_WARNING
 *   Logs interesting conditions which are probably not fatal
 * \def LOG_INFO
 *   Used for providing general useful information
 * \def LOG_DEBUG
 *   Debugging purposes only
 */
#define OUTPUTLEVEL LOG_DEBUG

/**
 * \brief Global indicating whether vertex programs
 * should indicate edges as directed or not.
 */
extern bool MTLT_DIRECTED_GRAPH;

// Project includes
#ifdef USING_GRAPHCHI
#include <graphchi_basic_includes.hpp>

// Wrapper for using GraphLab-style GAS programs in GraphChi.
#include "api/graphlab2_1_GAS_api/graphlab.hpp"

#include <vector>
#include <algorithm>
#include <errno.h>
#include <assert.h>

#include "io/stripedio.hpp"
#include "logger/logger.hpp"
#include "util/merge.hpp"
#include "util/ioutil.hpp"
#include "util/qsort.hpp"
#include "api/chifilenames.hpp"
#include "engine/auxdata/vertex_data.hpp"
#include <graphlab/parallel/mutex.hpp>

#else
#include <graphlab.hpp>
#endif

#endif



