/**
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
 * @section DESCRIPTION
 *
 * Generic graph kernel for computing only half a graph kernel.
 */
#ifndef HALF_GRAPH_KERNEL_HPP
#define HALF_GRAPH_KERNEL_HPP

#include "sparse_vector.hpp"
#include "graphlabchi.hpp"

template<class VertexType, class EdgeType>
class HalfGraphKernel {

    public:

    typedef graphlab::distributed_graph<VertexType, EdgeType> GraphType;

        /**
         * Constructs a new half graph kernel for comparing the two graphs g1 and g2.
         */
        HalfGraphKernel(GraphType& _g1, graphlab::distributed_control& _dc, std::string _engine_type, graphlab::command_line_options& _clopts) : 
            g1(_g1), dc(_dc), engine_type(_engine_type), clopts(_clopts) {}

        /**
         * Calculates the normalized vector the half-kernel.
         */
        virtual std::vector<SparseVector> calculate(size_t num_graphs) = 0;

    protected:
    GraphType& g1;
    graphlab::distributed_control& dc;
    graphlab::command_line_options& clopts;
    std::string engine_type;

};

#endif
