import random
import sys
import time
import math
import numpy as np
from collections import defaultdict

def main():
	
	#There are currently 214 514 nodes in outputfile
	#There are currently 289 432 sources in outputfile
	#Malin had ca 180 000 nodes and 37 000 sources

	print "Program starting"
	if len(sys.argv) != 2:
		print "usage: walks_outputfile "
		sys.exit(1)

	best_accuracy = -1.0
	best_f1score = -1.0
	walks_location = sys.argv[1]	
	walk_count_full = dict()
	#READ FROM FILE
	nodes = []
	labels = dict()
	walks_outputfile = open(walks_location, "r")
	while True:
            	line = walks_outputfile.readline()
		#print "newline"
		#print line
		if line == "":
			break	
		
		line = line.split()
		float_line = map(float,line)
		#print float_line
		node = int(float_line[0])
		nodes.append(node)
		walk_count_full[node] = dict()

		float_line.pop(0) #the node
		labels[node] =  int(float_line[0])
		float_line.pop(0) #the label
		num_sources = int(float_line[0])

		i = 0
		while i < (len(float_line)):
			source = int(float_line[i])
			to_node = int(float_line[i+1])
			walk_prob = float_line[i+2]			

			if source != -1:
				if source not in walk_count_full[node]:
					walk_count_full[node][source] = dict()

				#print "Source and node " + str(source) + " " + str(node)
				walk_count_full[node][source][to_node] = walk_prob
			i = i+3	
	
	walks_outputfile.close()
	#print walk_count_full
	#print labels
	#print nodes
		
	##THE DIFFERENT THRESHOLDS TO BE TESTED
	test_list = [i*0.01 for i in range(101)]
	#test_list = [0.0]
	print "Tested thresholds " + str(test_list)  
	#print walk_count_full

	for threshold in test_list:

		true_pos = 0
		true_neg = 0
		false_pos = 0
		false_neg = 0
	
		for node in walk_count_full:
			#print "Current node " + str(node)
			walk_count = dict()
			for wc in walk_count_full[node]:
				walk_count[wc] = walk_count_full[node][wc].copy()
			#print "walk count " + str(walk_count)				

			for start_node in walk_count:
				to_remove = []
				for to_node in walk_count[start_node]:
					#LEQ only for the special case when threshold = 0.00000
					if walk_count[start_node][to_node] <= threshold:
						to_remove.append(to_node)
				for remove in to_remove:
					del walk_count[start_node][remove]			
	
			#print walk_count
			

			#Just see how many neighbours one node has if that node has the same as the number
			#of nodes in walk_count then the graph is connected, otherwise it is not. 
			#the choice of node should not influence the result.
			cluster = set([])
			if len(walk_count) == 0:
				print "no walks done since 1 source"		
	
			else: 
				for wc in walk_count:
					#cluster = set(get_new_neighbours(wc, walk_count, [wc]))
					cluster = get_new_neighbours_no_recursion(wc,walk_count)
					break #Only do for the first node

			#print "Clusters " + str(cluster)		
			
			#Check if the cluster is the whole graph
			#Second condition not needed
			if len(cluster) == len(walk_count) or len(walk_count) == 0:
				predicted_label = -1
			else:
				predicted_label = 1

			label = labels[node]

			#print "Predicted label " + str(predicted_label)
			#print "true label " + str(label)

			#True positive
			if predicted_label == label and label == 1:
				true_pos += 1
			#True negative
			elif predicted_label == label and label == -1:
				true_neg += 1
			#False negative
			elif predicted_label != label and label == 1:
				false_neg += 1
			#False positive
			elif predicted_label != label and label == -1:
				false_pos += 1
			else:
				print "ERROR:: NOT CLASSIFYABLE"


		acc_sum = 0
		acc_sum += true_pos
		acc_sum += true_neg
		num_to_ambiguate = len(nodes)
		
		accuracy = acc_sum/(float(num_to_ambiguate))	
		#print accuracy
		#Maybe should be done on F1 score

		precision = true_pos / float(true_pos + false_pos) 
		recall = true_pos / float( true_pos + false_neg   )
		f1score = 2*precision*recall / float(precision + recall)

		if f1score > best_f1score:
			best_f1score = f1score

		if accuracy > best_accuracy:
			best_true_pos = true_pos
			best_true_neg = true_neg
			best_false_pos = false_pos
			best_false_neg = false_neg
			best_accuracy = accuracy
			best_threshold = threshold

	print "The threshold with the best result: " + str(best_threshold) 
	print "True positives " + str(best_true_pos)
	print "True negatives " + str(best_true_neg)
	print "False positives " + str(best_false_pos)
	print "False negatives " + str(best_false_neg)
	print "Number to ambiguate " + str(num_to_ambiguate)
	print "Best accuracy " + str(best_accuracy)
	print "Best f1 score " + str(best_f1score)
	
def get_new_neighbours_no_recursion(node, walk_count):
	already_found = set([])
	stack = []
	stack.append(node)
	#print "Getting neighbours of node " + str(node)
	while len(stack) > 0:
		#print "already found " + str(already_found)
		#print "stack " + str(stack)
		n = stack[0]
		already_found.add(n)
		for neighbour in walk_count[n]:
			if neighbour not in already_found:
				stack.append(neighbour)
		stack.pop(0)
	return already_found



def get_new_neighbours(node, walk_count, already_found):
	#print "node "  + str(node)
	for neighbour in walk_count[node]:
		#print "nei " + str(neighbour)
		if already_found.count(neighbour) == 0:
			#print "new neig " + str(neighbour)
			already_found.append(neighbour)
			#print "found so far " + str(already_found)
			temp = get_new_neighbours (neighbour, walk_count, already_found) 
			#print "temp " + str(temp)
			
			already_found.extend(temp)
			already_found = list(set(already_found))
			#print "set list " + str(already_found)

	return already_found


if __name__=="__main__":
	main()
