#name_prediction.py
#This program predicts if a node is ambiguous only based on the names of 
#the nodes. A threshold is found and set using cross-validation.

import sys
import numpy as np
from collections import defaultdict

def main():

	if len(sys.argv) != 7:
		print "usage main name_frequency_path last_names_path male_names_path female_names_path labels_path outputfile"
		sys.exit(1)


	last_name_frequency_path = sys.argv[1]
	male_name_frequency_path = sys.argv[2]
	female_name_frequency_path = sys.argv[3]
	names_path = sys.argv[4]
	labels_path = sys.argv[5]
	outputfile_path = sys.argv[6]
	
	#Create the frequencty dict
	frequency_file = open(last_name_frequency_path, "r")
	last_name_freq = defaultdict(float)	
	while True:
		line = frequency_file.readline()
		if line == "":
			break
		line = line.split()
		last_name = line.pop(0)
		last_name = last_name.lower()
		freq = float(line.pop(0))
		
		last_name_freq[last_name] = freq


	#male first names
	frequency_file = open(male_name_frequency_path, "r")
	male_name_freq = defaultdict(float)
	while True:
		line = frequency_file.readline()
		if line == "":
			break
		line = line.split()
		first_name = line.pop(0)
		first_name = first_name.lower()
		freq = float(line.pop(0))
		male_name_freq[first_name] = freq


	#female first names
	frequency_file = open(female_name_frequency_path, "r")
	female_name_freq = defaultdict(float)
	while True:
		line = frequency_file.readline()
		if line == "":
			break
		line = line.split()
		first_name = line.pop(0)
		first_name = first_name.lower()
		freq = float(line.pop(0))
		female_name_freq[first_name] = freq


	#read the names & genders
	names_to_predict = []
	names_file = open(names_path, "r")
	while True:
		line = names_file.readline()
		if line == "":
			break
		line = line.split()
		first_name = line.pop(0)
		first_name = first_name.lower()
		if len(line) == 1: #In case the person does not have a last name
			last_name = ""
		else:
			last_name = line.pop(0)
		last_name = last_name.lower()
		gender = line.pop(0)
		if gender == "m":
			male = True
		elif gender == "f":
			male = False
		else:
			print "Error person neither male or female .. terminating"
			sys.exit(1)
		

		names_to_predict.append( (first_name, last_name, male)  )		
		
	#print names_to_predict	

	#read the labels
	labels = []
	labels_file = open(labels_path, "r")
	while True:
		line = labels_file.readline()
		if line == "":
			break
		label = int(line)
		labels.append(label)
	
	if len(labels) != len(names_to_predict):
		print "Not the same number of names as labels, something is wrong terminating..."
		sys.exit(1)
	num_to_classify = len(labels)
	
	

	#create data for classification
	pairs = []
	for flgname in names_to_predict:
		first_name = flgname[0]
		last_name = flgname[1]
		gender = flgname[2]
		l_freq = last_name_freq[last_name]
		if gender == True:
			f_freq = male_name_freq[first_name]
		else:
			f_freq = female_name_freq[first_name]
		
		full_name = first_name + " "
		full_name = full_name + last_name		


		pairs.append( (full_name, l_freq*1 + f_freq) )

	#print pairs

	#include labels
	indata = []
	for i in range(0,num_to_classify):
		indata.append( [pairs[i][0], pairs[i][1], labels[i]])
	#print indata


	#classify based on frequency
	thresholds = [i*0.01 for i in range(201)]
	#thresholds = [1.59]
	#print thresholds

	#Do cross-validation
	num_cross = 10
	t_len = num_to_classify / num_cross
	#print t_len

	k_fold = k_fold_cross_validation(indata, num_cross)
	cross_accuracies = []
		
	for cross_data in k_fold:

		best_tp = 0
		best_tn = 0
		best_fp = 0
		best_fn = 0		
		best_accuracy = 0.0
		best_threshold = -1

		v_data = cross_data[0]
		t_data = cross_data[1]
		#print "t_data"
		#print t_data
		#print "v_data"
		#print v_data
			
		for threshold in thresholds:
			tp = 0
			tn = 0
			fp = 0
			fn = 0
			i = 0

			for data in t_data:
				if data[1] > threshold:
					prediction = 1
				else:
					prediction = -1
				
				#True positive
				if prediction == 1 and data[2] == 1:
					tp += 1
				#True negative
				elif prediction == -1 and data[2] == -1:
					tn += 1
				#False positive
				elif prediction == 1 and data[2] == -1:
					fp += 1
					#print "False positive " + str(pair)

				#False negative
				elif prediction == -1 and data[2] == 1:
					fn += 1
					#print "False negative " + str(pair)

				else:
					print "Something is seriously wrong here ..."		
					sys.exit(1)
			
					
				
			accuracy = (tp+tn) / float(len(t_data))
			if accuracy >= best_accuracy:
				best_tp = tp
				best_tn = tn
				best_fp = fp
				best_fn = fn
				best_accuracy = accuracy
				best_threshold = threshold

		#Find out the real accuracy with the best threshold
				
		for data in v_data:
			if data[1] > best_threshold:
				prediction = 1
			else:
				prediction = -1
			
			#True positive
			if prediction == 1 and data[2] == 1:
				best_tp += 1
			#True negative
			elif prediction == -1 and data[2] == -1:
				best_tn += 1
			#False positive
			elif prediction == 1 and data[2] == -1:
				best_fp += 1
				#print "False positive " + str(pair)
			#False negative
			elif prediction == -1 and data[2] == 1:
				best_fn += 1
				#print "False negative " + str(pair)

			else:
				print "Something is seriously wrong here ..."		
				sys.exit(1)
		

		real_accuracy = (best_tp + best_tn) / float(num_to_classify)
		cross_accuracies.append(real_accuracy)
		print "\n"
		print "Best threshold: " + str(best_threshold)
		print "Best true positives: " + str(best_tp)
		print "Best true negatives: " + str(best_tn)
		print "Best false positives: "	+ str(best_fp)
		print "Best false negatives: " + str(best_fn)
		print "Real accuracy: "	+ str(real_accuracy)



	if len(cross_accuracies) != num_cross:
		print "BIG BIG ERROR, not same number of cross validations performed as it should be"

	final_accuracy = 0.0
	for acc in cross_accuracies:
		final_accuracy += acc
	final_accuracy /= float(len(cross_accuracies))

	print "\n"
	print "Program complete"
	#print "Best threshold: " + str(best_threshold)
	#print "Best true positives: " + str(best_tp)
	#print "Best true negatives: " + str(best_tn)
	#print "Best false positives: "	+ str(best_fp)
	#print "Best false negatives: " + str(best_fn)
	print "Cross-validation accuracy: " + str(final_accuracy)


def k_fold_cross_validation(X, K):

	for k in xrange(K):
		training = [x for i, x in enumerate(X) if i % K != k]
		validation = [x for i, x in enumerate(X) if i % K == k]
		yield training, validation
    
if __name__=="__main__":
	main()
