/**
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
 * @section DESCRIPTION
 *
 * Edge used for calculating shortest paths.
 */
#ifndef SHORTEST_PATH_EDGE_HPP
#define SHORTEST_PATH_EDGE_HPP

#include "graphlabchi.hpp"
#include "shortest_path_types.hpp"

/**
 * \brief The length associated with the edge.
 */
class edge_data
#ifdef USING_GRAPHCHI
		: graphlab::IS_POD_TYPE
#endif
		  {
public:

  distance_type length;

  edge_data(distance_type length = 1) : length(length) { }

#ifdef USING_GRAPHCHI
#else
  void save(graphlab::oarchive& oarc) const {
		  oarc << length;
  }
  void load(graphlab::iarchive& iarc) {
		  iarc >> length;
  }
#endif
}; // end of edge data

#endif
