/**
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
 *
 * @section DESCRIPTION
 *
 * A graph containing shortest path information in each vertex.
 */
#ifndef SHORTEST_PATH_GRAPH_HPP
#define SHORTEST_PATH_GRAPH_HPP

#include "graphlabchi.hpp"
#include "helper.hpp"
#include "shortest_path_program.hpp"

class sp_vertex_data
#ifdef USING_GRAPHCHI
		: graphlab::IS_POD_TYPE
#endif
		  {
public:

  double sp_dist;
  sp_vertex_data(double _sp_dist = -1) : sp_dist(_sp_dist) { }

#ifdef USING_GRAPHCHI
#else
  void save(graphlab::oarchive& oarc) const {
		  oarc << sp_dist;
  }
  void load(graphlab::iarchive& iarc) {
		  iarc >> sp_dist;
  }
#endif

}; // end of sp_vertex data

class sp_edge_data
#ifdef USING_GRAPHCHI
		: graphlab::IS_POD_TYPE
#endif
		  {
public:

  double sp_dist;
  sp_edge_data(double _sp_dist = -1) : sp_dist(_sp_dist) { }

#ifdef USING_GRAPHCHI
#else
  void save(graphlab::oarchive& oarc) const {
		  oarc << sp_dist;
  }
  void load(graphlab::iarchive& iarc) {
		  iarc >> sp_dist;
  }
#endif
};

typedef graphlab::distributed_graph<vertex_data, edge_data> base_graph_type;
typedef graphlab::distributed_graph<sp_vertex_data, sp_edge_data> sp_internal_graph_type;

base_graph_type::vertex_id_type next_id;
graphlab::mutex next_id_mutex;

void create_sp_vertex(base_graph_type::vertex_type& base_vertex);

sp_internal_graph_type* temp_graph;

void create_sp_graph(sp_internal_graph_type* sp_graph_p, base_graph_type* base_graph_p) {

    temp_graph = sp_graph_p;

    next_id = 0;
    next_id_mutex = graphlab::mutex();

    base_graph_p->transform_vertices(create_sp_vertex);

    temp_graph = 0;
}

void create_sp_vertex(base_graph_type::vertex_type& base_vertex) {

    sp_map_type source_dists = base_vertex.data().source_dists;

    FOR_EACH(sp_map_type, source_dists, it) {

        distance_type sp_dist = it->second;

        next_id_mutex.lock();
        base_graph_type::vertex_id_type new_id = next_id++;
        next_id_mutex.unlock();

        // Add vertex to this SP graph.
        temp_graph->add_vertex(new_id, sp_vertex_data(sp_dist));
    }
}

#endif

