/**
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
 * @section DESCRIPTION
 *
 * Shortest paths utility functions.
 */
#ifndef SHORTEST_PATH_UTILITY_HPP
#define SHORTEST_PATH_UTILITY_HPP

#include "graphlabchi.hpp"
#include "shortest_path_types.hpp"

/**
 * \brief Get the other vertex in the edge.
 */
template<class VertexType, class EdgeType>
inline VertexType
get_other_vertex(const EdgeType& edge,
		const VertexType& vertex) {
	return vertex.id() == edge.source().id() ? edge.target() : edge.source();
}

template <typename VertexDataType>
std::string get_vertex_sp_string(graphlab::vertex_id_type vertex_id, VertexDataType &v_data) {
    std::stringstream strm;
    strm << vertex_id << " ==> \t";
    FOR_EACH(sp_map_type, v_data.source_dists, it) {
        graphlab::vertex_id_type from_id = it->first;
        distance_type dist = it->second;
        strm << "(" << from_id << ":" << dist << ") ";
    }
    strm << std::endl;
    return strm.str();
}

/*
 * We want to save the final graph so we define a write which will be
 * used in graph.save("path/prefix", shortest_path_writer()) to save the graph.
 */
struct shortest_path_writer {
  std::string save_vertex(sp_graph_type::vertex_type v) {
    return get_vertex_sp_string<vertex_data>(v.id(), v.data());
  }
  std::string save_edge(sp_graph_type::edge_type e) { return ""; }
};


#endif

