/**
 * @file
 * @author  Tommi Kerola <kerola@student.chalmers.se>
 * @author  Linus Hermansson <hlinus@student.chalmers.se>
 * @version 1.0
 *
 * @section LICENSE
 *
 * Copyright [2013] [Tommi Kerola, Linus Hermansson]
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an "AS
 *  IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 *  express or implied.  See the License for the specific language
 *  governing permissions and limitations under the License.
 *
 * @section DESCRIPTION
 *
 * Vertex used for calculating shortest paths.
 */
#ifndef SHORTEST_PATH_VERTEX_HPP
#define SHORTEST_PATH_VERTEX_HPP

#include "graphlabchi.hpp"
#include "shortest_path_types.hpp"

typedef float distance_type;
typedef std::map<graphlab::vertex_id_type, distance_type> sp_map_type;

/**
 * \brief The current distance to the vertex from each source vertex.
 */
class vertex_data
#ifdef USING_GRAPHCHI
: graphlab::IS_POD_TYPE
#endif
{
    public:
        size_t cluster_id;
        bool distinguished_vertex;

        sp_map_type source_dists;
        bool has_sent_own_id;
        vertex_data(size_t _cluster_id = 0, bool _distinguished_vertex = false) : cluster_id(_cluster_id), distinguished_vertex(_distinguished_vertex), has_sent_own_id(false) { }

#ifdef USING_GRAPHCHI
#else
        void save(graphlab::oarchive& oarc) const {
            oarc << source_dists << cluster_id << distinguished_vertex;
        }
        void load(graphlab::iarchive& iarc) {
            iarc >> source_dists >> cluster_id >> distinguished_vertex;
        }
#endif

}; // end of vertex data

#endif


